/* $Id: $ */

/* Copyright (C) 1996, 1997 Bjrn Kirkerud
 * Department of Informatics, University of Oslo.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "config.h"

#ifdef HAVE_DMEDIA_AUDIO_H
#include <dmedia/audio.h>
#include <math.h>
#include <stdio.h>

/* Global variables: */

ALconfig    config;
ALport	    audiooutPort;
ALport	    audioinPort;
short	    buf[100000];




/*
 * GetAudioHardwareInputRate:  get audio hardware input sampling rate
*/

long GetAudioHardwareInputRate(void)
{
long buffer[6];

/* acquire state variables of audio hardware */
buffer[0] = AL_INPUT_RATE;
buffer[2] = AL_INPUT_SOURCE;
buffer[4] = AL_DIGITAL_INPUT_RATE;
ALgetparams(AL_DEFAULT_DEVICE, buffer, 6);

/* for input sources microphone or line and input rate not AES word clock */
if	((buffer[3] != AL_INPUT_DIGITAL)&&(buffer[1] > 0))
    return (buffer[1]);

/* for input rate AES word clock and machine has ability to read digital 
    input sampling rate, return AL_DIGITAL_INPUT_RATE */
else if (ALgetdefault(AL_DEFAULT_DEVICE, AL_DIGITAL_INPUT_RATE) >= 0) 
    return (buffer[5]);

/* could not determine sampling rate */
else
    return (AL_RATE_UNDEFINED);
}   /* ---- end GetAudioHardwareInputRate() ---- */


/*
 * GetAudioHardwareOutputRate	get audio hardware output sampling rate	
*/
long GetAudioHardwareOutputRate(void)
{
long	buffer[4];

buffer[0] = AL_OUTPUT_RATE;
buffer[2] = AL_DIGITAL_INPUT_RATE;
ALgetparams(AL_DEFAULT_DEVICE, buffer, 4);

/* return output rate (Hertz) */
if	(buffer[1] > 0) 
    return (buffer[1]);

/* when output rate is input rate, get input rate */
else if (AL_RATE_INPUTRATE == buffer[1])
    return (GetAudioHardwareInputRate());

/* for input rate AES word clock and machine has ability to read digital 
    input sampling rate, return AL_DIGITAL_INPUT_RATE */
else if (ALgetdefault(AL_DEFAULT_DEVICE, AL_DIGITAL_INPUT_RATE) >= 0) 
    return (buffer[3]);

/* could not determine sampling rate */
else
    return (AL_RATE_UNDEFINED);
} /* ---- end GetAudioHardwareOutputRate() ---- */


initiateaudio()
{
  double  samplingRate;
  ALseterrorhandler(0);
  samplingRate = GetAudioHardwareOutputRate();
  if (samplingRate == AL_RATE_UNDEFINED)
      samplingRate = 44100;
  config = ALnewconfig();
  ALsetqueuesize(config, samplingRate);
  ALsetchannels(config, AL_MONO);
}

closeaudio()
{
  ALfreeconfig(config);
}



audioout(float *out, int length, int mult)
{ int i, p, m;
  for (p = 0; p < length; p++)
    {
     for (m = 0; m < mult; m++)
       {
        buf[p*mult + m] = 32767.0*out[p];
       }
    };
  audiooutPort = ALopenport("outport", "w", config);
  ALwritesamps(audiooutPort, buf, mult*length);
  /* allow audio to drain from port */
  while (ALgetfilled(audiooutPort) > 0) 
      sginap(1); 
  ALcloseport(audiooutPort);
}


audioin(float *intab, int length, int mult)
{
  int i, p;
  audioinPort  = ALopenport("inport", "r", config);
  ALreadsamps(audioinPort, buf, mult*length);
  ALcloseport(audioinPort);
  for (p = 0; p < length; p++)
  {
    intab[p] = ((float) buf[p]/32767.0);
  };
}


/* ******************************************************************
 *  getinputrate:	return Audio Hardware input sampling rate
 * ****************************************************************** */

long getinputrate()
{
long buf[6];

buf[0] = AL_INPUT_RATE;
buf[2] = AL_INPUT_SOURCE;
buf[4] = AL_DIGITAL_INPUT_RATE;
ALgetparams(AL_DEFAULT_DEVICE, buf, 6);

/* we are clocked off digital input. find real input rate, if possible. */
if	((buf[1] == AL_RATE_AES_1)||(buf[3] == AL_INPUT_DIGITAL)) 
    {
    if (ALgetdefault(AL_DEFAULT_DEVICE, AL_DIGITAL_INPUT_RATE) >= 0) 
	return (buf[5]);
    }
/* input rate is Hz and we're using an analog input */
else if (buf[1] > 0) 
    return (buf[1]);

return (AL_RATE_UNDEFINED);
}

/* ******************************************************************
 *  getoutputrate:	return Audio Hardware output sampling rate
 * ****************************************************************** */
long getoutputrate()
{
long buf[4];

buf[0] = AL_OUTPUT_RATE;
buf[2] = AL_DIGITAL_INPUT_RATE;
ALgetparams(AL_DEFAULT_DEVICE, buf, 4);

/* output rate is in Hz -- return it */
if (buf[1] > 0) 
    return (buf[1]);

/* output rate is logical -- track down what it means */
else 
    {
    if	    (buf[1] == AL_RATE_AES_1) 
	{
	/* we are clocked off of digital input. find
	 * real input rate, if system supports this ability.
	 * Otherwise, return AL_RATE_UNDEFINED */
	if (ALgetdefault(AL_DEFAULT_DEVICE,AL_DIGITAL_INPUT_RATE) >= 0) 
	    return (buf[3]);
	}
    else if (buf[1] == AL_RATE_INPUTRATE) 
	return (getinputrate());
    return (AL_RATE_UNDEFINED);
    }
} /* --------------------- end GetOutputRate() --------------- */

#else

/*
 * GetAudioHardwareInputRate:  get audio hardware input sampling rate
*/

long GetAudioHardwareInputRate()
{}


/*
 * GetAudioHardwareOutputRate	get audio hardware output sampling rate	
*/
long GetAudioHardwareOutputRate()
{}


initiateaudio()
{}

closeaudio()
{}



audioout(out, length, mult)
  float *out; 
  int length, mult;
{}


audioin(intab, length, mult)
  float *intab;
  int length, mult;
{}


/* ******************************************************************
 *  getinputrate:	return Audio Hardware input sampling rate
 * ****************************************************************** */

long getinputrate()
{}

/* ******************************************************************
 *  getoutputrate:	return Audio Hardware output sampling rate
 * ****************************************************************** */
long getoutputrate()
{}

#endif
