/* $Id: $ */

/* Copyright (C) 1995, 1997 Knut Omang
 * Department of Informatics, University of Oslo.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Socket interface library from slib, SCI interface library
 * modified for use for Simula Web connections ( 1995 Knut Omang) */
#include "config.h"

#ifdef HAVE_NETINET_TCP_H
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/tcp.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <netinet/in.h>

/* Interface to Simula */

int openhttp(char* server)
{
  return ip_connect(server,0,"http");
}


int openhttplink(char* server,char* link)
{
  int fd,len;
  char buf[100];
  fd = ip_connect(server,0,"http");

  sprintf(buf,"GET %s HTTP/1.0\nAccept: */*\n\n",link);
  len = strlen(buf);
  write(fd,buf,len);
  return fd;
}


int cwrite(int fd, char* buf, int len)
{
  int stat;
  printf("%d,len=%d:%s:\n",fd,len,buf); fflush(stdout);
  stat = write(fd,buf,len);

  printf("OK\n"); fflush(stdout);
  return stat;
}


/* IP functions from slib library code */

static void check(int b, char* m)
{
  if (b) return;
  fprintf(stderr,"%s\n",m);
  exit(1);
}


int ip_initconn(char* hostname,int port,char* portname, 
		struct sockaddr_in* server)
{
  struct hostent *hp;
  struct servent *serv;

  check(port == 0 || portname == 0,"supply either port or portname");
  check(port != 0 || portname != 0,"supply either port or portname");

  /* create socket */

  if (portname)
    {
      serv = getservbyname(portname,"tcp");
      if (!serv)
        {
          fprintf(stderr,"getservbyname: tcp/%s: not a service.\n",portname);
          exit(1);
        }
      port = ntohs(serv->s_port);
    }

  if (!hostname) 
    {
      fprintf(stderr, "ip_initconn: Called with nil hostname\n");
      exit(EINVAL);
    }
  hp = gethostbyname(hostname);

  if (hp == 0)
    {
      fprintf(stderr,"%s : unknown host\n", hostname);
      return -1;
    }

  memset((char*)server,0,sizeof(server));
  server->sin_port = htons(port);

  memcpy((char*)&(server->sin_addr), (char*)hp->h_addr, hp->h_length);
  server->sin_family = AF_INET;
  return 0;
}


int ip_doconn(struct sockaddr_in* server)
{
  int sock;

  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0)
    {
      perror("opening socket");
      exit(1);
    }
  
  if (connect(sock, (struct sockaddr*)server, sizeof(*server)) < 0)
    {
      if (errno == ECONNREFUSED)
	{
	  close(sock);
	  return -1;
	}
      perror("ip_doconn");
      exit(errno);
    }
  return sock;
}


int ip_connect(char* hostname,int port,char* portname)
{
  struct sockaddr_in server;
  int stat = ip_initconn(hostname,port,portname,&server);
  if (stat < 0) return stat;
  return ip_doconn(&server);
}
#endif
