/* $Id: $ */

/* Copyright (C) 1988-1997 Bjrn Kirkerud
 * Department of Informatics, University of Oslo.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* *******************************************************************
	xinterface.c  :  C interface to X routines for WINDOWTOOLS

	Bjorn Kirkerud: Institute for Informatics,
	                University of Oslo
 
 *********************************************************************  */
#include "config.h"

#ifdef X_DISPLAY_MISSING
#else

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/cursorfont.h>
#include <X11/extensions/shape.h>

#ifdef HAVE_X11_XPM_H
#include <X11/xpm.h>
#endif

#define max_points 2000

#define TRUE 1
#define FALSE 0

#define rast_width 4
#define rast_height 4

static char rast0_bits[] = {0x00, 0x00, 0x00, 0x00};
static char rast1_bits[] = {0x01, 0x00, 0x00, 0x00};
static char rast2_bits[] = {0x01, 0x00, 0x04, 0x00};
static char rast3_bits[] = {0x05, 0x00, 0x04, 0x00};
static char rast4_bits[] = {0x05, 0x00, 0x05, 0x00};
static char rast5_bits[] = {0x05, 0x02, 0x05, 0x00};
static char rast6_bits[] = {0x05, 0x02, 0x05, 0x08};
static char rast7_bits[] = {0x05, 0x0a, 0x05, 0x08};
static char rast8_bits[] = {0x05, 0x0a, 0x05, 0x0a};
static char rast9_bits[] = {0x07, 0x0a, 0x05, 0x0a};
static char rast10_bits[] = {0x07, 0x0a, 0x0d, 0x0a};
static char rast11_bits[] = {0x0f, 0x0a, 0x0d, 0x0a};
static char rast12_bits[] = {0x0f, 0x0a, 0x0f, 0x0a};
static char rast13_bits[] = {0x0f, 0x0b, 0x0f, 0x0a};
static char rast14_bits[] = {0x0f, 0x0b, 0x0f, 0x0e};
static char rast15_bits[] = {0x0f, 0x0f, 0x0f, 0x0e};
static char rast16_bits[] = {0x0f, 0x0f, 0x0f, 0x0f};

static char *rastptr[] = {
  rast16_bits, rast15_bits, rast14_bits, rast13_bits, rast12_bits,
  rast11_bits, rast10_bits, rast9_bits, rast8_bits, rast7_bits,
  rast6_bits, rast5_bits, rast4_bits, rast3_bits, rast2_bits,
  rast1_bits, rast0_bits
};

typedef struct{
   int type;
   unsigned long serial;
   Bool send_event;
   Display *display;
   Window window;
   Window root;
   Window subwindow;
   Time time;
} XTimeEvent;

typedef struct{
   int type;
   unsigned long serial;
   Bool send_event;
   Display *display;
   Window window;
   Window root;
   Window subwindow;
   Time time;
   int x, y;
} XCoordEvent;

/* Global variables */
XPoint      points[max_points];

Display    *dpy = NULL;            /* X server connection */
int         screen;         /* Default screen */
int         depth;          /* Default depth */
Visual     *visual;         /* Default visual */
Colormap    colormap;       /* Default colormap */ 
XFontStruct *fontstruct;    /* Font descriptor */
XGCValues   gcv;            /* Struct for creating GC */
XEvent      event;          /* Event received */
XAnyEvent   anyevent;       /* Event received */
XSizeHints  xsh;            /* Size hints for window manager */
char       *geomSpec;       /* Window geometry string */
XSetWindowAttributes xswa;  /* Temporary Set Window Attribute struct */
Window      alarmwindow;

XColor      defaultcolors[256];  /* Defaultcolors when colormap is full */
char        clientcminstalled = FALSE;

int         xshapesupport = 0, xshape_event, xshape_error;

/******************************************************
 *
 *  0: General utility functions:
 *
 ******************************************************/

void sleepus(n_usecs)
int n_usecs;
{
  /* Sleep n_usecs microseconds.
     (Not accurate for values below about 10000.) */

  struct timeval tval;
  
  tval.tv_sec  = n_usecs / 1000000;
  tval.tv_usec = n_usecs % 1000000;
  select(0, NULL, NULL, NULL, &tval);
}


void systemcall(s)
char *s;
{
  system(s);
}


/*******************************************************
 *
 *  1: Procedure for opening the display: 
 *
 *******************************************************/

Display *xopendisplay(dname) char *dname;
{
  dpy = XOpenDisplay(dname);
  if ( dpy == (Display *) NULL ) {
      
    }
  else {
/*    XSynchronize (dpy, True); */
/* Synchronous behaviour (nice when debugging) */
    screen = DefaultScreen(dpy);
    depth = DefaultDepth(dpy, screen);
    colormap = DefaultColormap(dpy, screen);
#ifdef HAVE_XSHAPEQUERYEXTENSION
    xshapesupport = XShapeQueryExtension (dpy, &xshape_event, &xshape_error);
#endif
  }
  return dpy;
}


/*******************************************************
 *
 *  2: Procedures for handling windows:
 *
 *******************************************************/

Window xrootwindow()
{
   return DefaultRootWindow(dpy);
}

Window xcreatesimplewindow(parentID, x, y, width, height, 
                           borderwidth, borderpix, backpix) 
    Window parentID; int x, y, width, height, borderwidth, borderpix, backpix;
{
     return XCreateSimpleWindow(dpy, parentID,
     			      x, y, width, height,
     			      borderwidth, borderpix, backpix);
}


Window xcreatewindow(parentID, window_name)
    Window parentID; char *window_name;
{
  int WindowID;
  WindowID = XCreateSimpleWindow(dpy, parentID,
     			      0, 0, 1, 1, 1, 
     			      BlackPixel(dpy, screen),
     			      WhitePixel(dpy, screen));
  XSetStandardProperties(dpy, 
                         WindowID, window_name, 
                         window_name, 0, 
                         None, 0, &xsh);
  return WindowID;
}


Window xcreatepopup(parentID, window_name)
    Window parentID; char *window_name;
{
  int WindowID;
  WindowID = XCreateSimpleWindow(dpy, parentID,
     			      0, 0, 1, 1, 1, 
     			      BlackPixel(dpy, screen),
     			      WhitePixel(dpy, screen));
  XSetStandardProperties(dpy, 
                         WindowID, window_name, 
                         window_name, 0, 
                         None, 0, &xsh);
   xswa.override_redirect = TRUE;
   XChangeWindowAttributes(dpy, WindowID, CWOverrideRedirect, &xswa);

  return WindowID;
}


void xsetstandardproperties(windowID, window_name, icon_name, icon_pixmap)
    Window windowID; char *window_name, *icon_name; Pixmap icon_pixmap;
{
    XSetStandardProperties(dpy, 
                           windowID, window_name, 
                           icon_name, icon_pixmap, 
                           None, 0, &xsh);
}

void xmapwindow(WindowNumber)
    int WindowNumber;
{
    XMapWindow(dpy, WindowNumber);
    XFlush(dpy);
}

void xunmapwindow(WindowNumber)
    int WindowNumber;
{
    XUnmapWindow(dpy, WindowNumber);
    XFlush(dpy);
}

void xclearwindow(WindowNumber)
    int WindowNumber;
{
    XClearWindow(dpy, WindowNumber);
    XFlush(dpy);
}

void xdestroywindow(WindowNumber)
    int WindowNumber;
{
    XDestroyWindow(dpy, WindowNumber);
    XFlush(dpy);
}

void xresizewindow(WindowNumber, width, height)
    int WindowNumber, width, height;
{
    XResizeWindow(dpy, WindowNumber, width, height);
    XFlush(dpy);
}

void xmovewindow(WindowNumber, x, y)
    int WindowNumber, x, y;
{
    XMoveWindow(dpy, WindowNumber, x, y);
    XFlush(dpy);
}

void xraisewindow(WindowNumber)
    int WindowNumber;
{
    XRaiseWindow(dpy, WindowNumber);
    XFlush(dpy);
}

void xsetwindowborderwidth(WindowNumber, bw)
    int WindowNumber, bw;
{
    XSetWindowBorderWidth(dpy, WindowNumber, bw);
    XFlush(dpy);
}

/*******************************************************
 *
 *  3: Procedures for drawing:  
 *
 *******************************************************/

void xdrawstring(WindowNumber, gc1, x, y, t)
   int WindowNumber, gc1, x, y; char t[];
{
   XDrawString(dpy, WindowNumber, (GC)gc1, x, y, t, strlen(t));
   XFlush(dpy);
}

void xdrawimagestring(WindowNumber, gc1, x, y, t)
   int WindowNumber, gc1, x, y; char t[];
{
   XDrawImageString(dpy, WindowNumber, (GC)gc1, x, y, t, strlen(t));
   XFlush(dpy);
}

void xdrawpoint(WindowNumber, gc, x, y) 
   int WindowNumber, gc, x, y;
{
   XDrawPoint(dpy, WindowNumber, (GC)gc, x, y);
   XFlush(dpy);
}

void xfillpolygon(WindowNumber, gc, x, y, n, mode) 
   int WindowNumber, gc, x[], y[], n, mode;
{
  int i, dmode;
  if (n > max_points) n = max_points;
  for( i = 0; i < n+1; i++)
    {
      points[i].x = x[i];
      points[i].y = y[i];
    }
   if (mode > 0) dmode = CoordModeOrigin;
   else          dmode = CoordModePrevious;
   XFillPolygon(dpy, WindowNumber, (GC)gc, points, n, Complex, dmode);
   XFlush(dpy);
}

void xfillrectangle(WindowNumber, gc, x, y, width, height) 
   int WindowNumber, gc, x, y, width, height;
{
   XFillRectangle(dpy, WindowNumber, (GC)gc, x, y, width, height);
   XFlush(dpy);
}

void xcleararea(WindowNumber, x, y, width, height) 
   int WindowNumber, x, y; unsigned int width, height;
{
   XClearArea(dpy, WindowNumber, x, y, width, height, False);
   XFlush(dpy);
}

void xdrawline(WindowNumber, gc, x1, y1, x2, y2) 
   int WindowNumber, gc, x1, y1, x2, y2;
{
   XDrawLine(dpy, WindowNumber, (GC)gc, x1, y1, x2, y2);
   XFlush(dpy);
}

void xrootline(gc, x1, y1, x2, y2) GC gc; int x1, y1, x2, y2;
{
   XDrawLine(dpy, DefaultRootWindow(dpy), gc, x1, y1, x2, y2);
   XFlush(dpy);
}

void xdrawrectangle(WindowNumber, gc, x, y, width, height) 
   int WindowNumber, gc, x, y, width, height;
{
   XDrawRectangle(dpy, WindowNumber, (GC)gc, x, y, width, height);
   XFlush(dpy);
}

void xdrawarc(WindowNumber, gc, x, y, width, height, angle1, angle2) 
   int WindowNumber, gc, x, y, width, height, angle1, angle2;
{
   XDrawArc(dpy, WindowNumber, (GC)gc, x, y, width, height, angle1, angle2);
   XFlush(dpy);
}

void xfillarc(WindowNumber, gc, x, y, width, height, angle1, angle2) 
   int WindowNumber, gc, x, y, width, height, angle1, angle2;
{
   XFillArc(dpy, WindowNumber, (GC)gc, x, y, width, height, angle1, angle2);
   XFlush(dpy);
}



/*******************************************************
 *
 *  4: Procedures for handling fonts:
 *
 *******************************************************/

int xfontid(font_struct) XFontStruct *font_struct; 
{
    return font_struct->fid;
}

int xfontheight(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_bounds.ascent + font_struct->max_bounds.descent); 
}

int xfontascent(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_bounds.ascent);
}

int xfontdescent(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_bounds.descent);
}

int xfontmaxwidth(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_bounds.width);
}

int xfontminwidth(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->min_bounds.width);
}

int xfontlbearing(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_bounds.lbearing);
}

int xfontrbearing(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_bounds.rbearing);
}

int xfontminchar(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->min_char_or_byte2);
}

int xfontmaxchar(font_struct) XFontStruct *font_struct; 
{
    return (font_struct->max_char_or_byte2);
}

int xcharlbearing(font_struct, rank) XFontStruct *font_struct; int rank; 
{
    return (font_struct->per_char[rank].lbearing);
}

int xcharrbearing(font_struct, rank) XFontStruct *font_struct; int rank; 
{
    return (font_struct->per_char[rank].rbearing);
}

int xcharwidth(font_struct, rank) XFontStruct *font_struct; int rank; 
{
    return (font_struct->per_char[rank].width);
}

int xcharascent(font_struct, rank) XFontStruct *font_struct; int rank; 
{
    return (font_struct->per_char[rank].ascent);
}

int xchardescent(font_struct, rank) XFontStruct *font_struct; int rank; 
{
    return (font_struct->per_char[rank].descent);
}

XFontStruct *xloadqueryfont(fn)
    char fn[];
{
    return XLoadQueryFont(dpy, fn);
}

void xdestroyfont(fn) XFontStruct *fn;
{
  XFreeFont(dpy,fn);
}

int xtextwidth(font_struct, t)
    XFontStruct *font_struct; char t[];
{
    return XTextWidth(font_struct, t, strlen(t));
}


Cursor xarrowcursor()
{
  return XCreateFontCursor(dpy,XC_top_left_arrow);
}

Cursor xxtermcursor()
{
  return XCreateFontCursor(dpy, XC_xterm);
}

void xsetcursor(WindowID,CursorID) int WindowID,CursorID;
{
  XDefineCursor(dpy, WindowID, CursorID); 
}

/*******************************************************
 *
 *  5: Procedures for synchronization:
 *
 *******************************************************/

void xflush()
{
   XFlush(dpy);
}


/*******************************************************
 *
 *  6: Procedures for event-handling:
 *
 *******************************************************/


void xselectinput(WindowNumber, EventMask)
    int WindowNumber, EventMask;
{
    XSelectInput(dpy, WindowNumber, EventMask);
}


XEvent *xeventpointer()
{
   return &event;
}


void alarmfunc(sig) int sig;
{
  event.xclient.window = alarmwindow;
  event.xclient.message_type = 6; /* Magic numbers, just to distinguish */
  event.xclient.format = 32;      /* from other clientmessages */
  XSendEvent(dpy, alarmwindow, 0, ExposureMask, &event);
  XFlush(dpy);
}

void xalarmclock(WindowNumber, time) int WindowNumber, time;
{
   alarmwindow = WindowNumber;
   signal(SIGALRM, alarmfunc);
   alarm(time);
}

int xpending()
{
   return XPending(dpy);
}

void xnextevent(event) XEvent *event; 
{
   XNextEvent(dpy, event);
}

Bool xchecktypedeventwindow(event) XAnyEvent *event; 
{
   return XCheckTypedWindowEvent(dpy, event->window, event->type,
(XEvent*)event);
}

int xeventtype(event) XEvent *event; 
{
   return event->type;
}

int xeventtime(event) XTimeEvent *event; 
{
   return event->time;
}

int xeventx(event) XCoordEvent *event; 
{
   return event->x;
}

int xeventy(event) XCoordEvent *event; 
{
   return event->y;
}

int xeventbutton(event) XButtonEvent *event; 
{
   return event->button;
}

int xeventkeycode(event) XKeyEvent *event; 
{
   return event->keycode;
}

Bool xclientmessage(event) XEvent *event;
{
  return (event->xany.send_event
	  && event->xclient.message_type == 6
	  && event->xclient.format == 32);
}

Window xeventwindow(event) XAnyEvent *event; 
{
   return event->window;
}

/*******************************************************
 *
 *  7: Procedures to get various masks etc.:
 *
 *******************************************************/

int blackpixel()
{
    return BlackPixel(dpy, screen);
}

int whitepixel()
{
    return WhitePixel(dpy, screen);
}

int keypressmask()
{
    return KeyPressMask;
}

int buttonpressmask()
{
    return ButtonPressMask;
}

int buttonreleasemask()
{
    return ButtonReleaseMask;
}

int enterwindowmask()
{
    return EnterWindowMask;
}

int leavewindowmask()
{
    return LeaveWindowMask;
}

int pointermotionmask()
{
    return PointerMotionMask;
}

int structurenotifymask()
{
    return StructureNotifyMask;
}

int exposuremask()
{
    return ExposureMask;
}

int clientmessage()
{
    return ClientMessage;
}

int keypress()
{
    return KeyPress;
}

int buttonpress()
{
    return ButtonPress;
}

int buttonrelease()
{
    return ButtonRelease;
}

int enternotify()
{
    return EnterNotify;
}

int destroynotify()
{
    return DestroyNotify;
}

int configurenotify()
{
    return ConfigureNotify;
}

int leavenotify()
{
    return LeaveNotify;
}

int motionnotify()
{
    return MotionNotify;
}

int mapnotify()
{
    return MapNotify;
}

int expose()
{
    return Expose;
}



/*******************************************************
 *
 *  8: Various procedures:  
 *
 *******************************************************/

int xscreendepth()
{
  return depth;
}

void xwarppointer(WindowNumber, x, y)
   int WindowNumber, x, y;
{
   XWarpPointer(dpy, None, WindowNumber, 0, 0, 0, 0, x, y);
   XFlush(dpy);
}

Status xgetgeometry(d, root, x, y, width, height, borderwidth, depth)
   Window d; Window *root; int *x, *y;
   unsigned int *width, *height, *borderwidth, *depth;
{
  return XGetGeometry(dpy, d, root, x, y, width, height, borderwidth, depth);
}

Status xqueryparent(d,root,parent)
  Window d;Window *root,*parent;
{
	Window *children;  
	unsigned int nchilds;
	Status s;

	s = XQueryTree(dpy,d,root,parent,&children,&nchilds);
	XFree((void*)children);
	return s;
}

Bool xquerypointer(w, root, child, rootx, rooty, winx, winy, kbuttons)
   Window w; Window *root, *child; int *rootx, *rooty, *winx, *winy;
   unsigned int *kbuttons;
{
  return XQueryPointer(dpy, w, root, child, rootx, rooty, winx, winy,
                       kbuttons);
}

unsigned int xdisplaywidth()
{
  return DisplayWidth(dpy, screen);
}

unsigned int xdisplayheight()
{
  return DisplayHeight(dpy, screen);
}

unsigned int xdisplaywidthmm()
{
  return DisplayWidthMM(dpy, screen);
}

unsigned int xdisplayheightmm()
{
  return DisplayHeightMM(dpy, screen);
}

void xcopyarea(srcWindowID, dstWindowID, GCID, srcx, srcy, width, height, dstx, dsty) 
    int srcWindowID, dstWindowID, GCID, srcx, srcy, width, height, dstx, dsty;
{
  XCopyArea(dpy, srcWindowID, dstWindowID, (GC)GCID, 
            srcx, srcy, width, height, dstx, dsty); 
   XFlush(dpy);
}

int xlookupstring(event, keybuffer, bufferlength, keysym, status)
  XKeyEvent *event;
  char *keybuffer;
  int bufferlength;
  KeySym *keysym;
  XComposeStatus *status;
{
  return XLookupString(event, keybuffer, bufferlength, keysym, status);
}

void xsetstipple(gc, stipple) GC gc; Pixmap stipple;
{
  XSetFillStyle(dpy, gc, FillOpaqueStippled);
  XSetStipple(dpy, gc, stipple);
}

void xsetnostipple(gc) GC gc;
{
    XSetFillStyle(dpy, gc, FillSolid);
}

void xsetwindowbackground(WindowNumber, pix)
   int WindowNumber, pix;
{
    XSetWindowBackground(dpy, WindowNumber, pix);
}


void xsetwindowbackgroundpixmap(WindowNumber, pix)
     int WindowNumber, pix;
{
  XSetWindowBackgroundPixmap(dpy, WindowNumber, pix);
}

/* Assumes that this function is called exactly 17 times for each xdisplay */
Pixmap xrast ()
{
  static char **last = rastptr;

  return
    XCreateBitmapFromData (dpy, DefaultRootWindow (dpy), *(last++),
			   rast_width, rast_height);
}

int xreadbitmapfile(windowID, bitmap_file, 
                    bitmap_width, bitmap_height,bitmap,x_hot,y_hot)
    Window windowID; char *bitmap_file; 
    unsigned int *bitmap_width, *bitmap_height;
    Pixmap *bitmap;
    int  *x_hot, *y_hot; 
{
    int xr;

    xr = XReadBitmapFile(dpy, windowID, bitmap_file, 
                        bitmap_width, bitmap_height, bitmap,  x_hot, y_hot); 
    return xr;
}


void xsaveunder(windowID) Window windowID; 
{
   xswa.save_under = TRUE;
   XChangeWindowAttributes(dpy, windowID, CWSaveUnder, &xswa);
}

void xbackingstore(windowID) Window windowID; 
{
   xswa.backing_store = Always;
   XChangeWindowAttributes(dpy, windowID, CWBackingStore, &xswa);
}

GC xcreategc(wn) int wn;    
{
    gcv.foreground = BlackPixel(dpy, screen);
    gcv.background = WhitePixel(dpy, screen);
    gcv.graphics_exposures = FALSE;
    gcv.fill_style = FillOpaqueStippled;
    return XCreateGC(dpy, wn,
		     GCForeground | GCBackground | GCGraphicsExposures | GCFillStyle,
		     &gcv);
}

void xdestroygc(gc) GC gc;
{
  XFreeGC(dpy,gc);
}

void xfree(ID) int ID;
{
  XFree((void *)ID);
}

void xcopygc(sc,dst) GC sc,dst;
{
  XCopyGC(dpy,sc,GCFunction|GCForeground|GCBackground|GCLineWidth|
	  GCFillStyle|GCStipple|GCFont,dst);
}

void xinvertgc(gc) GC gc;
{
  XSetFunction(dpy,gc,GXinvert); 
}

void xxorgc(gc, d) GC gc; int d;
{
  XSetFunction(dpy,gc,GXxor);
  if (xscreendepth () == 1) {
    XSetForeground (dpy, gc, 1);
    XSetBackground (dpy, gc, 0);
  }
}

void xdrawinsubwindows (gc) GC gc;
{
  XSetSubwindowMode (dpy, gc, IncludeInferiors);
}

void xclipbysubwindows (gc) GC gc;
{
  XSetSubwindowMode (dpy, gc, ClipByChildren);
}

int xdefaultvisual()
{
  visual = DefaultVisual(dpy, screen);
  if (visual->class == PseudoColor)
    {
      return 1;
    }
  else
  if (visual->class == StaticGray)
    {
      return 2;
    }
  else
    {
      return 0;
    }
}

void xsetblackonwhite(gc) GC gc;
{
    XSetForeground(dpy, gc, BlackPixel(dpy, screen));
    XSetBackground(dpy, gc, WhitePixel(dpy, screen));
    XSetFillStyle(dpy, gc, FillSolid);
    XSetFunction(dpy,gc,GXcopy);
}

void xsetwhiteonblack(gc) GC gc;
{
    XSetForeground(dpy, gc, WhitePixel(dpy, screen));
    XSetBackground(dpy, gc, BlackPixel(dpy, screen));
    XSetFillStyle(dpy, gc, FillSolid);
    XSetFunction(dpy,gc,GXcopy);
}

void xsetforeground(gc, foreground) GC gc; unsigned long foreground;
{
    XSetForeground(dpy, gc, foreground);
}


void xsetbackground(gc, background) GC gc; unsigned long background;
{
    XSetBackground(dpy, gc, background);
}

void xsetlinewidth(gc, linewidth) GC gc; unsigned int linewidth;
{
    XSetLineAttributes(dpy, gc, linewidth, LineSolid, CapButt, JoinMiter);
}

void xsetfont(gc, font) GC gc; Font font;
{
    XSetFont(dpy, gc, font);
}

void xstorebytes(bytes, nbytes) char bytes[]; int nbytes;
{
  XStoreBytes(dpy, bytes, nbytes);
}

char *xfetchbytes(nbytes) int *nbytes;
{
  char *x;
  x = XFetchBytes(dpy, nbytes); 
  return x != NULL ? x : "";
}

int xgrabpointer(grab_window) 
          Window grab_window;
{

  return XGrabPointer(dpy,grab_window,False,
        ButtonPressMask|ButtonReleaseMask|
	EnterWindowMask|LeaveWindowMask|
        PointerMotionMask|ExposureMask| FocusChangeMask ,
        GrabModeAsync,GrabModeAsync,None,None,CurrentTime);
}

void xungrabpointer()
{
  XUngrabPointer(dpy,CurrentTime);
}

/* Color functions */

/* Return 24-bit RGB-value for colorname, -1 if invalid name */
int xparsecolor(spec) char *spec;
{
  XColor retdata;
  int st = XParseColor (dpy, colormap, spec, &retdata);
  unsigned short r, g, b;

  r = retdata.red>>8; g = retdata.green>>8; b = retdata.blue>>8;

/*  printf ("Color: %s, (%d, %d, %d)\n",
	  spec, r, g, b); */

  return st ? r*256*256+g*256+b: -1;
}

/* Allocate RO-pixel for RGB-value, return pixel-value */
int xalloccolor (RGB) int RGB;
{
  XColor color;
  int i;

  color.red = ((RGB>>16)&0xFF)<<8;
  color.green = ((RGB>>8)&0xFF)<<8;
  color.blue = (RGB&0xFF)<<8;

/*  printf ("Allocating (%d, %d, %d) - pixel %d",
	  color.red, color.green, color.blue, color.pixel); */

  if (XAllocColor (dpy, colormap, &color)) return color.pixel;
  else {
    if (!clientcminstalled) {
      for (i = 0; i < 256; i++) defaultcolors[i].pixel = i;
      XQueryColors (dpy, colormap, defaultcolors, 256);
      clientcminstalled = TRUE;
    }
    return -1;
  }
  
}

/* Set TRANSIENT_FOR-property (for popup-windows) */
void xsettransientforhint(win,tfwin) Window win, tfwin;
{
  XSetTransientForHint (dpy, win, tfwin);
}

/* Get RGB-values of pixel */
int xquerycolor(pix) int pix;
{
  XColor color;
  int r, g, b;

  if (clientcminstalled) {
    r = defaultcolors[pix].red>>8;
    g = defaultcolors[pix].green>>8;
    b = defaultcolors[pix].blue>>8;
  }
  else {
    color.pixel = pix;
    XQueryColor (dpy, colormap, &color);
    r = color.red>>8; g = color.green>>8; b = color.blue>>8;
  }
  return r*256*256+g*256+b;
}

void xgetdefault (progname, option, value) char *progname, *option, *value;
{
  char *c;

  for (c = progname; (*c) != ' ' && (*c) != '\0' && c-progname < 200-1; c++);
  (*c) = '\0';
  if ((c = strrchr (progname, '/'))) progname = c+1;
  c = XGetDefault (dpy, progname, option);
  if (c) strncpy (value, c, 200);
}

/* Allocates bitmaps as well as pixmaps */
int xcreatepixmap (w, h, d) int w, h, d;
{
  return XCreatePixmap (dpy, DefaultRootWindow (dpy), w, h, d);
}

void xfreepixmap (pm) Pixmap pm;
{
  if (pm != None) XFreePixmap (dpy, pm);
}

void xreadpixmap (t, d, p, m, w, h)
  char *t; Drawable d; Pixmap *p, *m; int *w, *h;
{
#ifdef HAVE_XPMREADFILETOPIXMAP
#ifdef HAVE_X11_XPM_H
  XpmAttributes attrs;
  attrs.visual = visual;
  attrs.colormap = colormap;
  attrs.depth = depth;
  attrs.closeness = 65535;
  attrs.valuemask = /*XpmVisual | XpmColormap | XpmDepth |*/ XpmCloseness;

  XpmReadFileToPixmap (dpy, d, t, p, m, &attrs);
  if (*p) {
    *w = attrs.width;;
    *h = attrs.height;
  } else 
#endif
#endif
    printf ("Could not load picture %s\n", t);
}

void xreadbitmap (t, d, p, w, h) char *t; Drawable d; Pixmap *p; int *w, *h;
{
  int dummy, width, height;
  if (XReadBitmapFile (dpy, d, t, &width, &height, p, &dummy, &dummy) == BitmapSuccess) {
    *w = width;
    *h = height;
  }
  else {
    *p = None;
    printf ("Could not load bitmap %s\n", t);
  }
}

void xsetclipmask (g, p) GC g; Pixmap p;
{
  XSetClipMask (dpy, g, p);
}

void xsetcliporigin (g, x, y) GC g; int x, y;
{
  XSetClipOrigin (dpy, g, x, y);
}

void xcopyplane(srcWindowID, dstWindowID, GCID, srcx, srcy, width, height, dstx, dsty) 
    Drawable srcWindowID, dstWindowID; GC GCID;
    int srcx, srcy, width, height, dstx, dsty;
{
  XCopyPlane (dpy, srcWindowID, dstWindowID, GCID, srcx, srcy, width, height,
	      dstx, dsty, 1);
}

void xshapecombinemask (w, m) Window w; Pixmap m;
{
#ifdef HAVE_XShapeCombineMask
  XShapeCombineMask (dpy, w, ShapeBounding, 0, 0, m, ShapeSet);
#endif
}

/* Does not use any Xlib-calls, may be moved later */
int xxorcombinepixels (a, b) int a, b;
{
  return a^b;
}
#endif
